////////////////////////////////////////////////////////////////////////////////
//
// Fichero:
//      CD3D9Trace.cpp
//
// Descripcin:
//      Implementacin de la clase CD3D9Trace
//
// Observaciones:
//      Se ha modificado GetDeviceCaps para que limite MaxTextureWidth y
//      MaxTextureHeight a 256
//
// Historial:
//      16/MAR/2003 Creacin
//
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////
#include <crtdbg.h> //_ASSERTE()

#include "CD3D9Trace.h"


////////////////////////////////////////////////////////////////////////////////
// Implementacin
////////////////////////////////////////////////////////////////////////////////

namespace D3D9Trace
{

//
// Constructor
//
    CD3D9Trace::CD3D9Trace(IDirect3D9 * const pv_pInterface)
        : m_pInterface(pv_pInterface)
    {
    }

//
// Set/Get
//
    inline void CD3D9Trace::SetInterface(IDirect3D9 * const pv_pInterface)
    {
        m_pInterface = pv_pInterface;
    }

    inline IDirect3D9 * CD3D9Trace::GetInterface(void) const
    {
        return(m_pInterface);
    }

//
// IUnknow
//
    HRESULT STDMETHODCALLTYPE CD3D9Trace::QueryInterface(REFIID riid, void** ppvObj)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->QueryInterface(riid, ppvObj) );
    }

    ULONG STDMETHODCALLTYPE CD3D9Trace::AddRef(void)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->AddRef() );
    }

    ULONG STDMETHODCALLTYPE CD3D9Trace::Release(void)
    {
        _ASSERTE(NULL != GetInterface() );

        ULONG l_ulRefCount = GetInterface()->Release();

        if (0 == l_ulRefCount){
             SetInterface(NULL);
             delete this;
            }

        return(l_ulRefCount);
    }

//
// IDirect3D9
//
    HRESULT STDMETHODCALLTYPE CD3D9Trace::CheckDepthStencilMatch(UINT Adapter, D3DDEVTYPE DeviceType, D3DFORMAT AdapterFormat, D3DFORMAT RenderTargetFormat, D3DFORMAT DepthStencilFormat)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->CheckDepthStencilMatch(Adapter, DeviceType, AdapterFormat, RenderTargetFormat, DepthStencilFormat) );
    }

    HRESULT STDMETHODCALLTYPE CD3D9Trace::CheckDeviceFormat(UINT Adapter, D3DDEVTYPE DeviceType, D3DFORMAT AdapterFormat, DWORD Usage, D3DRESOURCETYPE RType, D3DFORMAT CheckFormat)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->CheckDeviceFormat(Adapter, DeviceType, AdapterFormat, Usage, RType, CheckFormat) );
    }

    HRESULT STDMETHODCALLTYPE CD3D9Trace::CheckDeviceFormatConversion(UINT Adapter, D3DDEVTYPE DeviceType, D3DFORMAT SourceFormat, D3DFORMAT TargetFormat)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->CheckDeviceFormatConversion(Adapter, DeviceType, SourceFormat, TargetFormat) );
    }

    HRESULT STDMETHODCALLTYPE CD3D9Trace::CheckDeviceMultiSampleType(UINT Adapter, D3DDEVTYPE DeviceType, D3DFORMAT SurfaceFormat, BOOL Windowed, D3DMULTISAMPLE_TYPE MultiSampleType, DWORD* pQualityLevels)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->CheckDeviceMultiSampleType(Adapter, DeviceType, SurfaceFormat, Windowed, MultiSampleType, pQualityLevels) );
    }

    HRESULT STDMETHODCALLTYPE CD3D9Trace::CheckDeviceType(UINT Adapter, D3DDEVTYPE DeviceType, D3DFORMAT DisplayFormat, D3DFORMAT BackBufferFormat, BOOL Windowed)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->CheckDeviceType(Adapter, DeviceType, DisplayFormat, BackBufferFormat, Windowed) );
    }

    HRESULT STDMETHODCALLTYPE CD3D9Trace::CreateDevice(UINT Adapter, D3DDEVTYPE DeviceType, HWND hFocusWindow, DWORD BehaviorFlags, D3DPRESENT_PARAMETERS *pPresentationParameters, IDirect3DDevice9** ppReturnedDeviceInterface)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->CreateDevice(Adapter, DeviceType, hFocusWindow, BehaviorFlags, pPresentationParameters, ppReturnedDeviceInterface) );
    }

    HRESULT STDMETHODCALLTYPE CD3D9Trace::EnumAdapterModes(UINT Adapter, D3DFORMAT Format, UINT Mode, D3DDISPLAYMODE* pMode)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->EnumAdapterModes(Adapter, Format, Mode, pMode) );
    }

    UINT STDMETHODCALLTYPE CD3D9Trace::GetAdapterCount(VOID)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->GetAdapterCount() );
    }

    HRESULT STDMETHODCALLTYPE CD3D9Trace::GetAdapterDisplayMode(UINT Adapter, D3DDISPLAYMODE *pMode)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->GetAdapterDisplayMode(Adapter, pMode) );
    }

    HRESULT STDMETHODCALLTYPE CD3D9Trace::GetAdapterIdentifier(UINT Adapter, DWORD Flags, D3DADAPTER_IDENTIFIER9 *pIdentifier)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->GetAdapterIdentifier(Adapter, Flags, pIdentifier) );
    }

    UINT STDMETHODCALLTYPE CD3D9Trace::GetAdapterModeCount(UINT Adapter, D3DFORMAT Format)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->GetAdapterModeCount(Adapter, Format) );
    }

    HMONITOR STDMETHODCALLTYPE CD3D9Trace::GetAdapterMonitor(UINT Adapter)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->GetAdapterMonitor(Adapter) );
    }

    HRESULT STDMETHODCALLTYPE CD3D9Trace::GetDeviceCaps(UINT Adapter, D3DDEVTYPE DeviceType, D3DCAPS9 *pCaps)
    {
        _ASSERTE(NULL != GetInterface() );

//Modificado para mostrar un ejemplo del uso de esta clase
        HRESULT l_hResult = GetInterface()->GetDeviceCaps(Adapter, DeviceType, pCaps);

        pCaps->MaxTextureWidth  = 256;
        pCaps->MaxTextureHeight = 256;

        return(l_hResult);
//        return( GetInterface()->GetDeviceCaps(Adapter, DeviceType, pCaps) );
    }

    HRESULT STDMETHODCALLTYPE CD3D9Trace::RegisterSoftwareDevice(void *pInitializeFunction)
    {
        _ASSERTE(NULL != GetInterface() );

        return( GetInterface()->RegisterSoftwareDevice(pInitializeFunction) );
    }

}
